'use strict';

var path = require('path');
var fs = require('fs');
var replacepath = require('replacepath');

var unique = function(arr){
    var newarr = [];
    var temp = {};
    arr.forEach(function(val){
        if(!temp[val]){
            temp[val] = true
            newarr.push(val)
        }
    })
    return newarr
}

var isEmptyObject = function(obj){
    var name;
    for(name in obj){
        return false;
    }
    return true;
}

var inArray = function(str, arr){
    var len = arr.length;
    var i=0;
    for(i; i<len; i++){
        if(arr[i] == str){
            return true
        }
    }
    return false
}

module.exports = function(base, dirname, options){

    var cacheModules = {};

    function Module(attrs, deps){
        var mod = this;
        mod.id = attrs[0];
        mod.name = attrs[1];
        mod.deps = deps;
        mod.depmodules = {};
        mod.filename = path.basename(mod.id);
        mod.uri = mod.id.replace(new RegExp(mod.filename+'$'), '');
        mod.dir = attrs[2];
    }

    Module.prototype.load = function(entry){
        var mod = this;
        if(!/_module_\d+/.test(mod.name)){
            mod.url = mod.id + '.js';
            if(fs.existsSync(mod.url)){
                var buffer = fs.readFileSync(mod.url);
                var content = buffer.toString();
                var arrs = mod.getdeps(content);
                var name = mod.name;
                var relative = /^\.+\//.test(name);
                //不同目录中如果有相同模块名，如果以相对路径作为模块名引入，在合并后会造成模块名冲突
                //为了解决此问题将相对路径转换为完整的绝对路径作为模块id
                arrs[2].forEach(function(val, i){
                    var rep = val;
                    arrs[0].forEach(function(v, k){
                        if(/^\.+\//.test(v)){
                           rep = rep.replace(new RegExp(Module.replace(v), 'g'), replacepath(mod.uri.replace(base, '')+v))
                        }
                    })
                    content = content.replace(new RegExp(Module.replace(val), 'g'), rep)
                })

                if(!entry && relative){
                    name = replacepath(mod.dir.replace(base, '')+name)
                }

                if(!/Nui.define/.test(content)){
                    var vars = '\n\tvar module=this,require=module.require,imports=module.imports,renders=module.renders,extend=module.extend;';
                    content = 'Nui.define(function(){'+ vars +'\n\t'+ content.replace(/\n/g, '\n\t') +'\n});'
                }

                mod.content = content.replace(/Nui.define\((['"][^'"]+['"]\s*,)?/, "Nui.define('"+ name +"',")
                                     .replace(/renders\(\{((.|\s)*?)\}\)/g, function(str, code){
                	                     return 'renders(\''+code.replace(/'/g, '\\\'').replace(/([\r\n]+)(\s+)?/g, "'+''$1$2+'")+'\')'
                                     })
                mod.deps = arrs[0];
                mod.styles = arrs[1];
            }
        }
        return mod.resolve()
    }

    Module.prototype.resolve = function(){
        var mod = this;
        if(isEmptyObject(mod.depmodules)){
            mod.deps.forEach(function(val, i){
                var module = Module.getModule(val, [], mod.uri);
                mod.depmodules[val] = module.load(mod.entry)
            })
        }
        return mod
    }

    Module.prototype.getdeps = function(content){
        var mod = this;
        var deps = [];
        var alldeps = [];
        var styles = [];
        var depsMatchs = content.match(/Nui.define\((['"][^'"]+['"]\s*,)?\s*\[([^\[\]]+)\]/);
        var requireMatchs = content.match(/(require|extend|imports)\(['"][^'"]+['"]/g);
        if(depsMatchs){
            var match = depsMatchs[2].split(',');
            match.forEach(function(val, i){
                val = val.replace(/^\s+|\s+/g, '').replace(/['"]/g, '');
                if(!/^https?:\/\//.test(val) && !inArray(val, options.ignore)){
                    deps.push(val)
                }
            })
            alldeps.push(depsMatchs[0])
        }
        if(requireMatchs){
            requireMatchs.forEach(function(val, i){
                if(/^(require|extend)/.test(val)){
                    var name = val.replace(/(require|extend)|[\('"]/g, '');
                    if(!/^https?:\/\//.test(name) && !inArray(name, options.ignore)){
                        alldeps.push(val);
                        deps.push(name)
                    }
                }
                else{
                    val = val.replace(/(imports)|[\('"]/g, '')
                    if(!/^https?:\/\//.test(val)){
                        styles.push(val)
                    }
                }
            })
        }
        return [unique(deps), unique(styles), unique(alldeps)]
    }

    Module.prototype.getContents = function(contents){
        var mod = this;
        if(!contents){
            contents = [];
        }
        contents.unshift(mod.content)
        if(mod.deps.length){
            for(var i in mod.depmodules){
                contents = mod.depmodules[i].getContents(contents)
            }
        }
        return contents
    }

    Module.prototype.loadstyles = function(styles){
        var mod = this;
        if(!styles){
            styles = [];
        }
        var arrs = [];
        if(mod.styles && mod.styles.length){
            mod.styles.forEach(function(val){
                arrs.push(Module.setId(val, mod.uri)[0]+'.css');
            })
        }
        styles = arrs.concat(styles);
        if(mod.deps.length){
            for(var i in mod.depmodules){
                styles = mod.depmodules[i].loadstyles(styles)
            }
        }

        return styles
    }

    //正则转义
    Module.replace = function(val){
        ['{', '}', '[', ']', '(', ')', '.'].forEach(function(v){
            val = val.replace(new RegExp('\\'+v, 'g'), '\\'+v)
        })
        return val
    }

    Module.setPath = function(id){
        var pathMatch = id.match(/\{([^\{\}]+)\}/);
        if(pathMatch){
            var _path = options.paths[pathMatch[1]];
            if(_path){
                id = id.replace(pathMatch[0], _path).replace(/(\.(js|css))?(\?[\s\S]*)?$/g, '');
            }
        }
        return id
    }

    Module.setId = function(id, uri){
        var name = id.replace(/(\.(js|css))?(\?[\s\S]*)?$/g, '');
        id = Module.setPath(options.alias[name] || name);
        if(!(/^[A-Z]:/i).test(id)){
            id = uri + name;
        }
        return [replacepath(id), name, uri]
    }

    Module.getModule = function(id, deps, uri){
        var arr = Module.setId(id, uri);
        var mod = cacheModules[arr[0]];
        if(mod){
            return mod
        }
        return (cacheModules[arr[0]] = new Module(arr, deps))
    }

    Module.id = 0;

    Module.load = function(id, callback){
        var mod = Module.getModule('_module_'+Module.id++, [id], dirname);
        mod.entry = true;
        callback(mod.load())
    }

    Module.getStyles = function(arr){
        var arrs = [];
        arr.forEach(function(val){
            if(fs.existsSync(val)){
                var buffer = fs.readFileSync(val);
                var content = buffer.toString();
                arrs.push(content);
            }
        })
        return arrs
    }

    return Module
}
